@@ -260,24 +260,27 @@ put_legacy_biomes_value <- function(db, x, z, dimension, value,
     mountain_meadow = 186L,
     lush_caves = 187L,
     dripstone_caves = 188L,
-    stony_peaks = 189L
+    stony_peaks = 189L,
+    deep_dark = 190,
+    mangrove_swamp = 191,
+    cherry_grove = 192,
+    pale_garden = 193
 )
 
-# Some biomes don't have a color in BDS, and their color was copied from
-# Java/Amidst. Ocean colors in BDS are very similar or identical, and their
-# colors were copied as well.
-# Amidst Colors: https://git.io/JYD2c
+# Cubiome Colors
+# https://github.com/Cubitect/cubiomes/blob/master/util.c
+
 .BIOME_COLORS <- c( # nolint: object_name_linter
     plains = "#8db360",
     desert = "#fa9418",
     extreme_hills = "#606060",
     forest = "#056621",
-    taiga = "#0b6659",
+    taiga = "#0b6a5f",
     swampland = "#07f9b2",
     river = "#0000ff",
-    hell = "#ff0000",
+    hell = "#572526",
     the_end = "#8080ff",
-    legacy_frozen_ocean = "#9090a0",
+    legacy_frozen_ocean = "#7070d6",
     frozen_river = "#a0a0ff",
     ice_plains = "#ffffff",
     ice_mountains = "#a0a0a0",
@@ -288,69 +291,73 @@ put_legacy_biomes_value <- function(db, x, z, dimension, value,
     forest_hills = "#22551c",
     taiga_hills = "#163933",
     extreme_hills_edge = "#72789a",
-    jungle = "#537b09",
+    jungle = "#507b0a",
     jungle_hills = "#2c4205",
-    jungle_edge = "#628b17",
+    jungle_edge = "#60930f",
     stone_beach = "#a2a284",
     cold_beach = "#faf0c0",
-    birch_forest = "#307444", # color was missing
-    birch_forest_hills = "#1F5F32", # color was missing
+    birch_forest = "#307444",
+    birch_forest_hills = "#1f5f32",
     roofed_forest = "#40511a",
     cold_taiga = "#31554a",
     cold_taiga_hills = "#243f36",
     mega_taiga = "#596651",
     mega_taiga_hills = "#454f3e",
-    extreme_hills_plus_trees = "#507050",
+    extreme_hills_plus_trees = "#5b7352",
     savanna = "#bdb25f",
     savanna_plateau = "#a79d64",
     mesa = "#d94515",
     mesa_plateau_stone = "#b09765",
     mesa_plateau = "#ca8c65",
-    ocean = "#000070", # color changed
-    deep_ocean = "#000030", # color changed
-    warm_ocean = "#0000ac", # color changed
-    deep_warm_ocean = "#000050", # color changed
-    lukewarm_ocean = "#000090", # color changed
-    deep_lukewarm_ocean = "#000040", # color changed
-    cold_ocean = "#202070", # color changed
-    deep_cold_ocean = "#202038", # color changed
-    frozen_ocean = "#7070d6", # color changed
-    deep_frozen_ocean = "#404090", # color changed
-    bamboo_jungle = "#537b09",
-    bamboo_jungle_hills = "#2c4205",
-    sunflower_plains = "#8db360",
-    swampland_mutated = "#b2f907",
-    ice_plains_spikes = "#d2ffff",
-    cold_taiga_mutated = "#4a5531",
-    savanna_mutated = "#E5DA87", # color was missing
-    savanna_plateau_mutated = "#CFC58C", # color was missing
-    roofed_forest_mutated = "#1a5140",
-    desert_mutated = "#1894fa",
-    flower_forest = "#6a7425",
-    taiga_mutated = "#59660b",
-    jungle_mutated = "#097b53",
-    jungle_edge_mutated = "#178b62",
-    mesa_bryce = "#1545d9",
-    mesa_plateau_stone_mutated = "#6597b0",
-    mesa_plateau_mutated = "#658cca",
-    birch_forest_mutated = "#589C6C", # color was missing
-    birch_forest_hills_mutated = "#47875A", # color was missing
-    redwood_taiga_mutated = "#596651",
-    extreme_hills_mutated = "#606060",
-    extreme_hills_plus_trees_mutated = "#507050",
-    redwood_taiga_hills_mutated = "#3e4f45",
-    soulsand_valley = "#5b4538",
-    crimson_forest = "#941818",
-    warped_forest = "#167e86",
-    basalt_deltas = "#685f70",
-    lofty_peaks = "#909090",
-    snow_capped_peaks = "#ffccd2",
-    snowy_slopes = "#ffffff",
-    mountain_grove = "#0c0c0c",
-    mountain_meadow = "#60b38d",
-    lush_caves = "#097b53",
-    dripstone_caves = "#1545d9",
-    stony_peaks = "#606060"
+    ocean = "#000070",
+    deep_ocean = "#000030",
+    warm_ocean = "#0000ac",
+    deep_warm_ocean = "#000050",
+    lukewarm_ocean = "#000090",
+    deep_lukewarm_ocean = "#000040",
+    cold_ocean = "#202070",
+    deep_cold_ocean = "#202038",
+    frozen_ocean = "#7070d6",
+    deep_frozen_ocean = "#404090",
+    bamboo_jungle = "#849500",
+    bamboo_jungle_hills = "#5c6c04",
+    sunflower_plains = "#b5db88",
+    swampland_mutated = "#2fffda",
+    ice_plains_spikes = "#b4dcdc",
+    cold_taiga_mutated = "#597d72",
+    savanna_mutated = "#e5da87",
+    savanna_plateau_mutated = "#cfc58c",
+    roofed_forest_mutated = "#687942",
+    desert_mutated = "#ffbc40",
+    flower_forest = "#2d8e49",
+    taiga_mutated = "#339287",
+    jungle_mutated = "#78a332",
+    jungle_edge_mutated = "#88bb37",
+    mesa_bryce = "#ff6d3d",
+    mesa_plateau_stone_mutated = "#d8bf8d",
+    mesa_plateau_mutated = "#f2b48d",
+    birch_forest_mutated = "#589c6c",
+    birch_forest_hills_mutated = "#47875a",
+    redwood_taiga_mutated = "#818e79",
+    extreme_hills_mutated = "#888888",
+    extreme_hills_plus_trees_mutated = "#839b7a",
+    redwood_taiga_hills_mutated = "#6d7766",
+    soulsand_valley = "#4d3a2e",
+    crimson_forest = "#981a11",
+    warped_forest = "#49907b",
+    basalt_deltas = "#645f63",
+    lofty_peaks = "#dcdcc8",
+    snow_capped_peaks = "#b0b3ce",
+    snowy_slopes = "#c4c4c4",
+    mountain_grove = "#47726c",
+    mountain_meadow = "#60a445",
+    lush_caves = "#283c00",
+    dripstone_caves = "#4e3012",
+    stony_peaks = "#7b8f74",
+    deep_dark = "#031f29",
+    mangrove_swamp = "#2ccc8e",
+    cherry_grove = "#ff91c8",
+    pale_garden = "#696d95"
 )
 
 # invert the list
