@@ -34,9 +34,10 @@
 #include "nbt.h"
 #include "subchunk.h"
 #include "random.h"
+#include "actors.h"
 
 // for testing:
-SEXP bedrock_leveldb_test_cleanup() {
+SEXP bedrock_leveldb_test_cleanup(void) {
     bedrock_leveldb_cleanup();
     bedrock_leveldb_init();
     return R_NilValue;
@@ -122,20 +123,22 @@ static const R_CallMethodDef call_methods[] = {
     {"Cread_chunk_biomes", (DL_FUNC)&read_chunk_biomes, 1},
     {"Cwrite_chunk_biomes", (DL_FUNC)&write_chunk_biomes, 2},
 
+    {"rbedrock_actor_make_uniqueids", (DL_FUNC)&rbedrock_actor_make_uniqueids, 2},
+    {"rbedrock_actor_make_storagekeys", (DL_FUNC)&rbedrock_actor_make_storagekeys, 1},
+
     {"Cmcpe_random_seed", (DL_FUNC)&mcpe_random_seed, 1},
     {"Cmcpe_random_state", (DL_FUNC)&mcpe_random_state, 1},
     {"Cmcpe_random_get_uint", (DL_FUNC)&mcpe_random_get_uint, 2},
     {"Cmcpe_random_get_int", (DL_FUNC)&mcpe_random_get_int, 3},
     {"Cmcpe_random_get_double", (DL_FUNC)&mcpe_random_get_double, 1},
     {"Cmcpe_random_get_float", (DL_FUNC)&mcpe_random_get_float, 3},
-
     {"Cmcpe_random_create_seed", (DL_FUNC)&mcpe_random_create_seed, 6},
 
     {NULL, NULL, 0}};
 
-void rbedrock_init_nbt();
-void rbedrock_init_blocks();
-void rbedrock_init_random();
+void rbedrock_init_nbt(void);
+void rbedrock_init_blocks(void);
+void rbedrock_init_random(void);
 
 void attribute_visible R_init_rbedrock(DllInfo *info) {
     R_registerRoutines(info, NULL, call_methods, NULL, NULL);
